#    builder configuration editor
#    Copyright (C) 2009 Mike Reed <gedit@amadron.com>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.

#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.

#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

from gi.repository import GObject, Gio, Gtk, Gedit, PeasGtk
import logging
import logging.config
import os
import os.path

from Config import Config
from ConfigEditor import ConfigEditor

class BuilderApp(GObject.Object, Gedit.AppActivatable, PeasGtk.Configurable):
    __gtype_name__ = "BuilderApp"
    app = GObject.property(type=Gedit.App)
    
    # The only reason I have this BuilderApp class is I only wanted to
    # create the Config and ConfigEditor once.  Unfortunately Gedit seems
    # to create one of these classes every time you bring up the config
    # dialog.  So I have had to manually make these singletons
    config = None
    _configEditor = None

    def __init__(self):
        self._l = logging.getLogger("plugin.builder")

    def do_activate(self):
        GObject.Object.__init__(self)
        # Set up logging.  It was tempting to use gedit.debug but this
        # has three problems.
        # 1) Turning it on switches on all debug for the "PLUGIN" section
        #    creating an immense amount of irrelevent output
        # 2) You can't vary the level of debug, it is either on or off
        # 3) You have to import gedit which means some modules in this
        #    package that could be unit tested independently of gedit
        #    can't be if we use gedit.debug
        if os.path.exists('logging.conf'):
            try:
                logging.config.fileConfig('logging.conf')
                self._l = logging.getLogger('plugin.builder')
                self._l.info('Setup logging from logging.conf')
            except  Exception as inst:
                # Well, they wanted logging but something is wrong.
                # Give 'em everything
                self._init_logging(logging.DEBUG)
                self._l.error('Something wrong with logging.conf, '
                              'using DEBUG level logging to stderr')
                self._l.error(str(type(inst)))
                self._l.error(str(inst.args))
                self._l.error(str(inst))
        else:
            self._init_logging(logging.WARNING)

        self._l = logging.getLogger("plugin.builder")

        # data_dir for gedit eg /home/mike/.local/share/gedit/plugins/builder
        BuilderApp.config = Config(self.plugin_info.get_data_dir())

        # Create Gedit Builder plugin preferences dialog content.  This is 
        # the ConfigEditor and it is what the user uses to manipulate the
        # Config object
        BuilderApp._configEditor = ConfigEditor(BuilderApp.config)

        self._l.debug("Exiting")
        
    def _init_logging(self, level):
        f = logging.Formatter(
            '%(asctime)s: %(levelname)8s: %(module)s(%(lineno)d): %(funcName)s: %(message)s')
        h = logging.StreamHandler()
        h.setLevel(level)
        h.setFormatter(f)
        self._l = logging.getLogger("plugin.builder")
        self._l.addHandler(h)
        self._l.setLevel(level)
    
    def do_create_configure_widget(self):
        return BuilderApp._configEditor.dialog()
        
    def do_deactivate(self):
        self._l.debug("Entered")
        self._l.debug("Exiting")



