# -*- mode: Text; coding: utf-8; buffer-read-only: t -*-
# lineendingstyle.py generated automatically from lineendingstyle.py.in
#
# Line Ending Style, a plugin for Gedit 3
# Copyright (C) 2012  Daniel Trebbien <dtrebbien[AT]gmail.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import gettext
from gi.repository import GObject, Gdk, Gtk, Gedit
import inspect
import weakref

def get_trace_info(num_back_frames=0):
	frame = inspect.currentframe().f_back
	try:
		for i in range(num_back_frames):
			frame = frame.f_back

		filename = frame.f_code.co_filename

		# http://code.activestate.com/recipes/145297-grabbing-the-current-line-number-easily/
		lineno = frame.f_lineno

		func_name = frame.f_code.co_name
		try:
			# http://stackoverflow.com/questions/2203424/python-how-to-retrieve-class-information-from-a-frame-object
			cls_name = frame.f_locals["self"].__class__.__name__
		except:
			pass
		else:
			func_name = "%s.%s" % (cls_name, func_name)

		return (filename, lineno, func_name)
	finally:
		frame = None

# Bug 668924 - Make gedit_debug_message() introspectable <https://bugzilla.gnome.org/show_bug.cgi?id=668924>
try:
	debug_plugin_message = Gedit.debug_plugin_message
except:
	def debug_plugin_message(format_str, *format_args):
		filename, lineno, func_name = get_trace_info(1)
		Gedit.debug(Gedit.DebugSection.DEBUG_PLUGINS, filename, lineno, func_name)



try:
	GeditStatusComboBox = Gedit.StatusComboBox
except:
	class GeditStatusComboBox(Gtk.EventBox):
		"""A pure PyGObject port of the GeditStatusComboBox class.

		Current with `gedit-status-combo-box.h` and `gedit-status-combo-box.c` from
		the tree for commit f167eb6 <http://git.gnome.org/browse/gedit/tree/?id=f167eb694e9c98f76a19aad7480f571e92bd6603>"""

		# Uncommenting the following line results in `sys:1: Warning: cannot register existing type `GeditStatusComboBox'`
		#__gtype_name__ = "GeditStatusComboBox"

		__gproperties__ = {
			"label": (str, "LABEL", "The label", None, GObject.PARAM_READWRITE)
		}

		__gsignals__ = {
			"changed": (GObject.SIGNAL_RUN_LAST, None, (Gtk.MenuItem,))
		}

		ITEM_TEXT_KEY = "GeditStatusComboBoxItemText"
		ACTIVATE_HANDLER_ID_KEY = "GeditStatusComboBoxActivateHandlerID"

		@classmethod
		def new(cls, label_text):
			return GeditStatusComboBox(label_text)

		def __init__(self, label_text):
			Gtk.EventBox.__init__(self)

			debug_plugin_message("self=%r", self)
			weakself = weakref.ref(self)

			css = self.css = Gtk.CssProvider()
			data = "* {\n" + \
					"  -GtkButton-default-border : 0;\n" + \
					"  -GtkButton-default-outside-border : 0;\n" + \
					"  -GtkButton-inner-border: 0;\n" + \
					"  -GtkWidget-focus-line-width : 0;\n" + \
					"  -GtkWidget-focus-padding : 0;\n" + \
					"  padding: 0;\n" + \
					"}"
			try:
				css.load_from_data(data)
			except TypeError: # Issue #1
				css.load_from_data(data, -1)

			self.set_visible_window(True)

			frame = self.frame = Gtk.Frame()
			frame.show()

			button = self.button = Gtk.ToggleButton()
			button.set_relief(Gtk.ReliefStyle.NONE)
			button.show()

			self.__set_shadow_type()

			hbox = self.hbox = Gtk.Box.new(Gtk.Orientation.HORIZONTAL, 3)
			hbox.show()

			self.add(frame)
			frame.add(button)
			button.add(hbox)

			label = self.label = Gtk.Label.new("")
			label.show()
			label.set_single_line_mode(True)
			label.set_halign(Gtk.Align.START)

			hbox.pack_start(label, False, True, 0)

			item = self.item = Gtk.Label.new("")
			item.show()
			item.set_single_line_mode(True)
			item.set_halign(Gtk.Align.START)

			hbox.pack_start(item, True, True, 0)

			arrow = self.arrow = Gtk.Arrow.new(Gtk.ArrowType.DOWN, Gtk.ShadowType.NONE)
			arrow.show()

			hbox.pack_start(arrow, False, True, 0)

			menu = self.menu = Gtk.Menu()
			menu.attach_to_widget(self, GeditStatusComboBox.__menu_detached)

			button.connect("button-press-event", GeditStatusComboBox.__button_press_event, weakself)
			button.connect("key-press-event", GeditStatusComboBox.__key_press_event, weakself)
			menu.connect("deactivate", GeditStatusComboBox.__menu_deactivate, weakself)

			# Make it as small as possible.
			context = button.get_style_context()
			context.add_provider(css, Gtk.STYLE_PROVIDER_PRIORITY_APPLICATION)

			context = frame.get_style_context()
			context.add_provider(css, Gtk.STYLE_PROVIDER_PRIORITY_APPLICATION)

			self.set_label(label_text)

		def __del__(self):
			debug_plugin_message("self=%r", self)
			self.menu.detach()

		def __set_shadow_type(self):
			# This is a hack needed to use the shadow type of a statusbar.
			statusbar = Gtk.Statusbar()
			context = statusbar.get_style_context()

			shadow_type_val = GObject.Value()
			shadow_type_val.init(Gtk.ShadowType)
			context.get_style_property("shadow-type", shadow_type_val)
			self.frame.set_shadow_type(shadow_type_val.get_enum())

		def __menu_detached(self, menu):
			self.menu = None

		@classmethod
		def __menu_position_func(cls, menu, weakcombo):
			combo = weakcombo()
			request, _ = menu.get_toplevel().get_preferred_size()

			_, x, y = combo.get_window().get_origin()

			# Make the menu at least as wide as the widget.
			allocation = combo.get_allocation()
			if request.width < allocation.width:
				menu.set_size_request(allocation.width, -1)

			# Position it above the widget.
			y -= request.height

			return (x, y, False)

		def __show_menu(self, button, time):
			weakself = weakref.ref(self)
			menu = self.menu
			request, _ = menu.get_preferred_size()

			# Do something relative to our own height here. Maybe we can do better.
			allocation = self.get_allocation()
			max_height = allocation.height * 20

			if request.height > max_height:
				menu.set_size_request(-1, max_height)
				menu.get_toplevel().set_size_request(-1, max_height)

			menu.popup(None, None, GeditStatusComboBox.__menu_position_func, weakself, button, time)

			self.button.set_active(True)

			if hasattr(self, "current_item"):
				menu.select_item(self.current_item)

		@classmethod
		def __button_press_event(cls, widget, event, weakcombo):
			combo = weakcombo()
			if event.type == Gdk.EventType.BUTTON_PRESS and event.button == 1:
				combo.__show_menu(event.button, event.time)
				return True

			return False

		@classmethod
		def __key_press_event(cls, widget, event, weakcombo):
			combo = weakcombo()
			if event.keyval == Gdk.KEY_Return or event.keyval == Gdk.KEY_ISO_Enter or \
					event.keyval == Gdk.KEY_KP_Enter or event.keyval == Gdk.KEY_space or \
					event.keyval == Gdk.KEY_KP_Space:
				combo.__show_menu(1, event.time)
				return True

			return False

		@classmethod
		def __menu_deactivate(cls, menu, weakcombo):
			combo = weakcombo()
			combo.button.set_active(False)

		def do_changed(self, item):
			try:
				text = getattr(item, GeditStatusComboBox.ITEM_TEXT_KEY)
			except AttributeError:
				pass
			else:
				self.item.set_markup(text)
				self.current_item = item

		def get_label(self):
			return self.label.get_label()

		def set_label(self, label_text):
			if label_text:
				label_text = "  %s: " % label_text
			else:
				label_text = "  "
			self.label.set_markup(label_text)

		@classmethod
		def __item_activate(cls, item, weakcombo):
			combo = weakcombo()
			combo.set_item(item)

		def add_item(self, item, text):
			weakself = weakref.ref(self)
			self.menu.append(item)

			self.set_item_text(item, text)

			activate_handler_id = item.connect("activate", GeditStatusComboBox.__item_activate, weakself)
			setattr(item, GeditStatusComboBox.ACTIVATE_HANDLER_ID_KEY, activate_handler_id)

		def remove_item(self, item):
			try:
				activate_handler_id = getattr(item, GeditStatusComboBox.ACTIVATE_HANDLER_ID_KEY)
			except AttributeError:
				pass
			else:
				item.disconnect(activate_handler_id)
				delattr(item, GeditStatusComboBox.ACTIVATE_HANDLER_ID_KEY)
			delattr(item, GeditStatusComboBox.ITEM_TEXT_KEY)
			self.menu.remove(item)

		def get_items(self):
			return self.menu.get_children()

		def get_item_text(self, item):
			try:
				return getattr(item, GeditStatusComboBox.ITEM_TEXT_KEY)
			except AttributeError:
				return None

		def set_item_text(self, item, text):
			setattr(item, GeditStatusComboBox.ITEM_TEXT_KEY, text)

		def set_item(self, item):
			self.emit("changed", item)

		def get_item_label(self):
			return self.item

		def do_get_property(self, property):
			if property.name == "label":
				return self.get_label()
			else:
				raise AttributeError, "unknown property %s" % property.name

		def do_set_property(self, property, value):
			if property.name == "label":
				self.set_label(value)
			else:
				raise AttributeError, "unknown property %s" % property.name



class LineEndingStylePluginUI:
	ITEM_VALUE_KEY = "GeditLineEndingStylePluginUIItemValue"
	ITEM_ACTIVATE_HANDLER_ID_KEY = "GeditLineEndingStylePluginUIItemActivateHandlerID"
	NOTIFY_NEWLINE_TYPE_HANDLER_ID_KEY = "GeditLineEndingStylePluginUINotifyNewline-TypeHandlerID"
	NOTIFY_READ_ONLY_HANDLER_ID_KEY = "GeditLineEndingStylePluginUINotifyRead-OnlyHandlerID"

	def __init__(self, window):
		debug_plugin_message("self=%r, window=%r", self, window)
		self.window = window

	def __del__(self):
		debug_plugin_message("self=%r", self)

	def __tab_added(self, window, tab):
		self.__connect_document(tab.get_document())

	def __active_tab_changed(self, window, tab):
		self.__update_state_per_document(tab.get_document())

	def __tab_removed(self, window, tab):
		self.__disconnect_document(tab.get_document())

	def __activate_item(self, item):
		self.set_active_document_newline_type(getattr(item, LineEndingStylePluginUI.ITEM_VALUE_KEY))

	def merge(self):
		action_group = self.action_group = Gtk.ActionGroup("GeditLineEndingStylePluginActions")
		action_group.set_translation_domain("gedit-line-ending-style-plugin")
		_ = lambda s: gettext.dgettext("gedit-line-ending-style-plugin", s)

		window = self.window
		statusbar = window.get_statusbar()
		sb_combo = self.sb_combo = GeditStatusComboBox.new(None)

		entries = [
			("GeditLineEndingStylePluginStatusComboToLFItem", "Unix/Linux", "Switch to Unix/Linux-style line endings (LF)",
					Gedit.DocumentNewlineType.LF, "LF"),
			("GeditLineEndingStylePluginStatusComboToCRItem", "Mac OS Classic", "Switch to Mac OS Classic-style line endings (CR)",
					Gedit.DocumentNewlineType.CR, "CR"),
			("GeditLineEndingStylePluginStatusComboToCRLFItem", "Windows", "Switch to Windows-style line endings (CRLF)",
					Gedit.DocumentNewlineType.CR_LF, "CRLF")
		]
		for entry in entries:
			action = Gtk.Action(entry[0],
					_(entry[1]),
					_(entry[2]),
					None)
			action_group.add_action(action)
			item = action.create_menu_item()
			setattr(item, LineEndingStylePluginUI.ITEM_VALUE_KEY, entry[3])
			activate_handler_id = item.connect("activate", self.__activate_item)
			setattr(item, LineEndingStylePluginUI.ITEM_ACTIVATE_HANDLER_ID_KEY, activate_handler_id)
			sb_combo.add_item(item, _(entry[4]))

		sb_combo.show_all()
		statusbar.pack_end(sb_combo, False, True, 0)

		doc = window.get_active_document()
		if doc:
			self.__update_state_per_document(doc)

		self.tab_added_handler_id = window.connect("tab-added", self.__tab_added)
		self.active_tab_changed_handler_id = window.connect("active-tab-changed", self.__active_tab_changed)
		self.tab_removed_handler_id = window.connect("tab-removed", self.__tab_removed)

		for doc in window.get_documents():
			self.__connect_document(doc)

	def __notify_document_property(self, doc, pspec):
		self.__update_state_per_document(doc)

	def __connect_document(self, doc):
		"""Connects plugin-specific event handlers."""

		notify_newline_type_handler_id = doc.connect("notify::newline-type", self.__notify_document_property)
		setattr(doc, LineEndingStylePluginUI.NOTIFY_NEWLINE_TYPE_HANDLER_ID_KEY, notify_newline_type_handler_id)

		notify_read_only_handler_id = doc.connect("notify::read-only", self.__notify_document_property)
		setattr(doc, LineEndingStylePluginUI.NOTIFY_READ_ONLY_HANDLER_ID_KEY, notify_read_only_handler_id)

	def __disconnect_document(self, doc):
		"""Disconnects plugin-specific event handlers."""

		try:
			notify_read_only_handler_id = getattr(doc, LineEndingStylePluginUI.NOTIFY_READ_ONLY_HANDLER_ID_KEY)
		except AttributeError:
			pass
		else:
			doc.disconnect(notify_read_only_handler_id)
			delattr(doc, LineEndingStylePluginUI.NOTIFY_READ_ONLY_HANDLER_ID_KEY)

		try:
			notify_newline_type_handler_id = getattr(doc, LineEndingStylePluginUI.NOTIFY_NEWLINE_TYPE_HANDLER_ID_KEY)
		except AttributeError:
			pass
		else:
			doc.disconnect(notify_newline_type_handler_id)
			delattr(doc, LineEndingStylePluginUI.NOTIFY_NEWLINE_TYPE_HANDLER_ID_KEY)

	def __update_state_per_document(self, doc):
		sb_combo = self.sb_combo

		if doc:
			nl_type = doc.get_property("newline-type")

			for item in sb_combo.get_items():
				if getattr(item, LineEndingStylePluginUI.ITEM_VALUE_KEY) == nl_type:
					sb_combo.set_item(item)
					break

			sb_combo.show()

			b = not doc.get_readonly()
			sb_combo.set_sensitive(b)
		else:
			sb_combo.hide()

	def set_active_document_newline_type(self, nl_type):
		doc = self.window.get_active_document()
		if doc and doc.get_property("newline-type") != nl_type:
			doc.set_property("newline-type", nl_type)
			doc.set_modified(True)

	def update_ui(self):
		doc = self.window.get_active_document()
		self.__update_state_per_document(doc)

	def unmerge(self):
		window = self.window
		for doc in window.get_documents():
			self.__disconnect_document(doc)

		window.disconnect(self.tab_removed_handler_id); del self.tab_removed_handler_id
		window.disconnect(self.active_tab_changed_handler_id); del self.active_tab_changed_handler_id
		window.disconnect(self.tab_added_handler_id); del self.tab_added_handler_id

		sb_combo = self.sb_combo
		for item in sb_combo.get_items():
			try:
				activate_handler_id = getattr(item, LineEndingStylePluginUI.ITEM_ACTIVATE_HANDLER_ID_KEY)
			except AttributeError:
				pass
			else:
				item.disconnect(activate_handler_id)
				delattr(item, LineEndingStylePluginUI.ITEM_ACTIVATE_HANDLER_ID_KEY)

		Gtk.Container.remove(window.get_statusbar(), sb_combo)



class LineEndingStylePlugin(GObject.Object, Gedit.WindowActivatable):
	__gtype_name__ = "LineEndingStylePlugin"

	UI_KEY = "GeditLineEndingStylePluginUI"

	window = GObject.property(type=Gedit.Window)

	def __init__(self):
		GObject.Object.__init__(self)

		debug_plugin_message("self=%r", self)

	def __del__(self):
		debug_plugin_message("self=%r", self)

	def do_activate(self):
		window = self.window
		ui = LineEndingStylePluginUI(window)
		ui.merge()
		setattr(window, LineEndingStylePlugin.UI_KEY, ui)

	def do_deactivate(self):
		window = self.window
		try:
			ui = getattr(window, LineEndingStylePlugin.UI_KEY)
		except AttributeError:
			pass
		else:
			ui.unmerge()
			delattr(window, LineEndingStylePlugin.UI_KEY)

	def do_update_state(self):
		window = self.window
		try:
			ui = getattr(window, LineEndingStylePlugin.UI_KEY)
		except AttributeError:
			pass
		else:
			ui.update_ui()

gettext.bindtextdomain("gedit-line-ending-style-plugin", "/usr/share/locale")
