# -*- coding: utf-8 -*-

import gettext
#import locale
#from locale import gettext as _
import os

from gi.repository import GObject, Gtk, Gedit, PeasGtk, Gio

from common import ghelp, utils

CURRENT_DIR = os.path.dirname(__file__)
APP_NAME = 'scratchtab'
SCRATCH_FILE = os.path.join(CURRENT_DIR, 'scratchtab.txt')
LOC_PATH = os.path.join(CURRENT_DIR, 'data', 'locale')
SETTINGS = ghelp.GeditPluginSettings(CURRENT_DIR, 'scratchtab')

# translation

# OK
#t = gettext.translation(APP_NAME, LOC_PATH)
#_ = t.ugettext

# OK
gettext.find(APP_NAME, LOC_PATH)
gettext.install(APP_NAME, LOC_PATH, True)

# OK
#gettext.bindtextdomain(APP_NAME, LOC_PATH)
#gettext.textdomain(APP_NAME)
#gettext.install(APP_NAME, LOC_PATH)

# OK: mais la barre de menu Gedit est maintenant en anglais
#locale.setlocale(locale.LC_ALL,'')
#locale.bindtextdomain(APP_NAME, LOC_PATH)
#locale.textdomain(APP_NAME)

# KO: Ne fonctionne plus avec GTK 3
#Gtk.glade.bindtextdomain(APP_NAME, LOC_PATH)
#Gtk.glade.textdomain(APP_NAME)


class ScratchTabConfig(ghelp.GeditPluginConfig):
    ''' Objet de configuration du plugin '''
    
    def __init__(self):
        ghelp.GeditPluginConfig.__init__(self, APP_NAME, CURRENT_DIR, SETTINGS)
    
    def _update_widget(self):
        position = self._settings.get_string('position')
        
        widget = self._builder.get_object('side_panel')
        widget.set_data('prefname', 'position')
        if position == 'side':
            widget.set_active(True)
            
        widget = self._builder.get_object('bottom_panel')
        widget.set_data('prefname', 'position')
        if position == 'bottom':
            widget.set_active(True)


class ScratchTab(GObject.Object, Gedit.WindowActivatable, PeasGtk.Configurable, 
        ghelp.GeditPluginHelper):
    ''' Plugin mettant à disposition une feuille de brouillon. '''
    
    __gtype_name__ = 'ScratchTab'
    
    window = GObject.property(type=Gedit.Window)
    
    def __init__(self):
        ''' Initialisation du plugin. '''
        GObject.Object.__init__(self)
        ghelp.GeditPluginHelper.__init__(self)
    
    def _test(self):
        builder = Gtk.Builder()
        builder.set_translation_domain(APP_NAME)
        builder.add_objects_from_file(os.path.join(CURRENT_DIR, 'data', 'ui', 'config.glade'), ['config_dialog'])
        widget = builder.get_object('config_dialog')
        widget.show_all()
        
        
    def do_create_configure_widget(self):
        ''' Création du panneau de configuration. '''
        return ScratchTabConfig().get_widget()
    
    
    def do_activate(self):
        ''' Activation du plugin. '''

        # connecter SETTINGS pour changement
        SETTINGS.connect_change('position', self.on_position_changed)
        
        # créer le panneau
        builder = utils.get_gtk_builder(CURRENT_DIR, 'pane.glade', ['pane'])
        builder.connect_signals(self)
        self._pane = builder.get_object('pane')
        self._area = builder.get_object('area')
        
        self._append_text(self._load_file())
        
        # créer un item de popup réutilisable ?
        
        # autres objets
        self._position = SETTINGS.get_string('position')
        self._icon = Gtk.Image.new_from_stock(Gtk.STOCK_EDIT, Gtk.IconSize.MENU)
        
        # Connection de window
        self._connect(self.window, 'tab-added', self.on_tab_added)
        self._connect(self.window, 'tab-removed', self.on_tab_removed)
        
        # Ajouter le panneau
        self._insert_pane()
    
    
    def do_deactivate(self):
        ''' Désactivation du plugin '''
        self._disconnect_all()
        self._remove_pane()
        self._pane = None
        self._area = None

    
    def on_position_changed(self, settings, key, data=None):
        ''' Ecouteur de changement de position dans les préférences. '''
        position = SETTINGS.get_string('position')
        if position != self._position:
            self._remove_pane()
            self._position = position
            self._insert_pane()
    
    
    def on_tab_added(self, window, tab):
        ''' Ecouteur d'ajout d'onglet '''
        self._connect(tab.get_view(), 'populate-popup', self.on_populate_popup)
    
    
    def on_tab_removed(self, window, tab):
        ''' Ecouteur de suppression d'onglet '''
        self._disconnect(tab.get_view())
    
    
    def on_area_focus_out_event(self, widget, event, data=None):
        ''' Listen lost focus sur zone de texte
            TODO: [medium] ne sauvegarder que si le texte a été modifié
        '''
        self._save_file();
    
    
    def on_populate_popup(self, view, menu):
        ''' Ecouteur d'ouverture de menu contextuel '''
        doc = view.get_buffer();
        if doc.get_has_selection():
            item = self._get_popup_item()
            item.show()
            menu.prepend(item)
    
    
    def on_popup_item_activate(self, item, data=None):
        ''' Ecouteur de click sur l'item de menu ScratchTab '''
        view = self.window.get_active_view()
        doc = view.get_buffer()
        sel = doc.get_selection_bounds()
        text = doc.get_text(sel[0], sel[1], True)
        self._append_text(text)
    
    
    def _insert_pane(self):
        ''' Insérer le panneau. '''
        panel = getattr(self.window, 'get_' + self._position + "_panel")()
        panel.add_item(self._pane, "ScratchTab", "ScratchTab", self._icon)
    
    def _remove_pane(self):
        ''' Supprimer le panneau. '''
        panel = getattr(self.window, 'get_' + self._position + "_panel")()
        panel.remove_item(self._pane)

    
    def _get_popup_item(self):
        ''' build scratchtab item for popup menu
            @return {ImageMenuItem}
        '''
        item = Gtk.ImageMenuItem(_('send to ScratchTab'))
        item.set_image(Gtk.Image.new_from_stock(Gtk.STOCK_EDIT, Gtk.IconSize.MENU))
        item.connect('activate', self.on_popup_item_activate)
        
        return item
        
        
    def _append_text(self, text):
        ''' Ajouter un texte à la feuille de brouillon.
            @param {String} text Texte à ajouter.
        '''
        doc = self._area.get_buffer()
        doc.insert(doc.get_end_iter(), text)
        self._save_file()
        
    
    def _load_file(self):
        ''' Load scratch file '''
        if os.path.isfile(SCRATCH_FILE) is False:
            f = file(SCRATCH_FILE, 'rw')
        else:
            f = open(SCRATCH_FILE, 'r')
        text = f.read()
        f.close()
        return text

    def _save_file(self):
        ''' Save scratch file '''
        doc = self._area.get_buffer()
        start = doc.get_start_iter()
        end = doc.get_end_iter()
        f = open(SCRATCH_FILE, 'w')
        f.write(doc.get_text(start, end, True))
        f.close()

