# -*- coding: utf-8 -*-
'''
- current_dir/data/ui
- current_dir/data/schema
- current_dir/data/img
'''
import os
import subprocess

from gi.repository import Gio, Gtk

SCHEMA_NAME_PREFIX = 'org.gnome.gedit.plugins.'


def get_settings(current_dir, name):
    '''
    @param String current_dir Répertoire dans lequel cherché le schéma.
    @param String name Nom du projet utilisant le schéma.
    '''
    schema_name = SCHEMA_NAME_PREFIX + name
    schemas = Gio.Settings.list_schemas()
    if schema_name in schemas:
        return Gio.Settings.new(schema_name)
    dir_path = os.path.join(current_dir, 'data', 'schema')
    if os.path.isdir(dir_path):
        schema_source = Gio.SettingsSchemaSource.new_from_directory(
            dir_path, 
            Gio.SettingsSchemaSource.get_default(), 
            False
        )
        schema = schema_source.lookup(schema_name, False)
        return Gio.Settings.new_full(schema, None, None)        
    return None


def get_gtk_builder(current_dir, filename, object_ids, locale_domain=None):
    '''
    '''
    filename = os.path.join(current_dir, 'data', 'ui', filename)
    builder = Gtk.Builder()
    if locale_domain is not None:
        builder.set_translation_domain(locale_domain)
    try:
        count = builder.add_objects_from_file(filename, object_ids)
        assert(count > 0)
        return builder
    except Exception as e:
        print "Failed to load %s: %s." % (filename, str(e))
#        if CURRENT_LOGGER is not None:
#            CURRENT_LOGGER.error("Failed to load %s: %s." % (filename, str(e)))
        return None


def get_icon_from_image(current_dir, name):
    path = os.path.join(current_dir, 'data', 'img', name)
    icon = Gtk.Image.new_from_file(path)
    iconset = Gtk.IconSet.new_from_pixbuf(icon.props.pixbuf)
    return Gtk.Image.new_from_icon_set(iconset, Gtk.IconSize.MENU)


def get_cmd_output(cmd):
    '''Exécuter une commande et récupérer le résultat.
    @param  String cmd Commande à exécuter
    '''
    p = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE)
    stdout, stderr = p.communicate()
    return stdout


def doc_is_of_type(doc, lang_ids = (), uri = None, exts = ()):
    '''Vérifier le type d'un document
    @param GtkBuffer doc
    @param List [lang_ids=()] Liste des identifiants de langues recherchées
    @param String [uri=None] Uri de la ressource
    @param List [exts=()] Liste des extensions de fichier acceptées.
    @return Boolean
    '''
    if not doc:
        return False
        
    if doc.get_language() is not None and lang_ids is not None:
        lang = doc.get_language()
        if lang.get_id() in lang_ids:
            return True
    
    if uri is not None:
        return any(uri.endswith('.' + ext) for ext in exts)
    
    return False

    
def find_in_path(names):
    pathList = os.environ['PATH'].split(':')
    for path in pathList:
        for name in names:
            if os.path.exists(path + '/' + name):
                return name
    return None
