# -*- coding: utf-8 -*-

from gi.repository import Gtk
from gi.repository import WebKit
from gi.repository import GObject

''' Inpector's code come from webkit examples '''
class Inspector (Gtk.Window):
    def __init__ (self, inspector):
        """initialize the WebInspector class"""
        GObject.GObject.__init__(self)
        self.set_default_size(600, 480)
        self._web_inspector = inspector
        self._web_inspector.connect("inspect-web-view",
                                    self._inspect_web_view_cb)
        self._web_inspector.connect("show-window",
                                    self._show_window_cb)
        self._web_inspector.connect("attach-window",
                                    self._attach_window_cb)
        self._web_inspector.connect("detach-window",
                                    self._detach_window_cb)
        self._web_inspector.connect("close-window",
                                    self._close_window_cb)
        self._web_inspector.connect("finished",
                                    self._finished_cb)
        self.connect("delete-event", self._close_window_cb)

    def _inspect_web_view_cb (self, inspector, web_view):
        """Called when the 'inspect' menu item is activated"""
        scrolled_window = Gtk.ScrolledWindow()
        scrolled_window.props.hscrollbar_policy = Gtk.PolicyType.AUTOMATIC
        scrolled_window.props.vscrollbar_policy = Gtk.PolicyType.AUTOMATIC
        webview = WebKit.WebView()
        scrolled_window.add(webview)
        scrolled_window.show_all()
        self.add(scrolled_window)
        return webview

    def _show_window_cb (self, inspector):
        """Called when the inspector window should be displayed"""
        self.present()
        return True

    def _attach_window_cb (self, inspector):
        """Called when the inspector should displayed in the same
        window as the WebView being inspected
        """
        return False

    def _detach_window_cb (self, inspector):
        """Called when the inspector should appear in a separate window"""
        return False

    def _close_window_cb (self, inspector, view):
        """Called when the inspector window should be closed"""
        self.hide()
        return True

    def _finished_cb (self, inspector):
        """Called when inspection is done"""
        self._web_inspector = 0
        self.destroy()
        return False

